// BezTest.cpp : Defines the entry point for the application.
//

#include "BezTest.h"
#include "Bezier.h"
#include <Windows.h>
#include <WindowsX.h>
#include <tchar.h>
#include <vector>

#define NUM_STEPS 100
#define WND_WD 600
#define WND_HGT 600
#define RESET_BUTTON_ID (WM_APP + 101)

HINSTANCE hInst;								
ATOM				MyRegisterClass(HINSTANCE hInstance);
BOOL				InitInstance(HINSTANCE, int);
LRESULT CALLBACK	WndProc(HWND, UINT, WPARAM, LPARAM);

std::vector<POINT> g_points;

int APIENTRY _tWinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPTSTR lpCmdLine, int nCmdShow) {
	MSG msg;
	MyRegisterClass(hInstance);

	if (!InitInstance (hInstance, nCmdShow))
		return FALSE;

	while (GetMessage(&msg, NULL, 0, 0)) {
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}

	return (int) msg.wParam;
}

ATOM MyRegisterClass(HINSTANCE hInstance) {
	WNDCLASSEX wcex;
	wcex.cbSize = sizeof(WNDCLASSEX);

	wcex.style			= CS_HREDRAW | CS_VREDRAW;
	wcex.lpfnWndProc	= WndProc;
	wcex.cbClsExtra		= 0;
	wcex.cbWndExtra		= 0;
	wcex.hInstance		= hInstance;
	wcex.hIcon			= 0;
	wcex.hCursor		= LoadCursor(NULL, IDC_ARROW);
	wcex.hbrBackground	= (HBRUSH)(COLOR_WINDOW+1);
	wcex.lpszMenuName	= 0;
	wcex.lpszClassName	= _T("JoesBezierTest");
	wcex.hIconSm		= 0;

	return RegisterClassEx(&wcex);
}

BOOL InitInstance(HINSTANCE hInstance, int nCmdShow) {
   HWND hWnd;

   hInst = hInstance; 
   if ((hWnd = CreateWindow(_T("JoesBezierTest"), _T("Joe's Bezier Test"), 
						WS_OVERLAPPEDWINDOW, 50, 50, WND_WD, WND_HGT, NULL, NULL, hInstance, NULL)) == NULL)
      return FALSE;
   ShowWindow(hWnd, nCmdShow);
   UpdateWindow(hWnd);

   return TRUE;
}

void DrawControlPoints(HDC hdc) {
	if (g_points.size() > 0) {

		HBRUSH hbrOld;
		HBRUSH hbr = CreateSolidBrush(RGB(255,0,0));
		hbrOld = static_cast<HBRUSH>(SelectObject(hdc, hbr));
		for (size_t i = 0; i < g_points.size(); ++i) {
			POINT pt = g_points[i];
			Ellipse(hdc, pt.x - 3, pt.y - 3, pt.x + 3, pt.y + 3);
		}
		SelectObject(hdc, hbrOld);
		DeleteObject(hbr);
		
		HPEN hpen = CreatePen(PS_DOT, 1, RGB(255,0,0));
		HPEN hpenOld = static_cast<HPEN>(SelectObject(hdc, hpen));
		if ( g_points.size() > 1) {
			for(size_t i = 1; i <  g_points.size(); i++) {
				MoveToEx(hdc, g_points[i-1].x,  g_points[i-1].y, NULL);
				LineTo(hdc, g_points[i].x,  g_points[i].y);
			}
		}
		SelectObject(hdc, hpenOld);
		DeleteObject(hpen);
	}
}

int f2i(float v) {
	return static_cast<int>(v + 0.5f);
}

void DrawBezierCurve(HDC hdc) {

	std::wstring msg(_T("To draw a cubic bezier, define control points by clicking in this window 4 times..."));
	RECT r = {10,10,WND_WD-10,45};
	DrawText(hdc, msg.c_str(), msg.size(), &r, DT_SINGLELINE); 

	DrawControlPoints(hdc);

	// okay, draw the bezier...
	if (g_points.size() == 4) {
		HPEN hpen = CreatePen(PS_SOLID, 1, RGB(0,0,255));
		HPEN hpenOld = static_cast<HPEN>(SelectObject(hdc, hpen));

		Bezier bez(	static_cast<float>(g_points[0].x), static_cast<float>(g_points[0].y),
					static_cast<float>(g_points[1].x), static_cast<float>(g_points[1].y),
					static_cast<float>(g_points[2].x), static_cast<float>(g_points[2].y),
					static_cast<float>(g_points[3].x), static_cast<float>(g_points[3].y) );

		float delta = 1.0f / static_cast<float>(NUM_STEPS);
		MoveToEx(hdc, g_points[0].x, g_points[0].y, NULL);
		for (size_t i = 1; i <= NUM_STEPS; i++) {
			float t = static_cast<float>(i) * delta;
			std::pair<float,float> pt = bez.getPoint(t);
			LineTo(hdc, f2i(pt.first), f2i(pt.second));
		}
		
		SelectObject(hdc, hpenOld);
		DeleteObject(hpen);
	}
}

void HandleClick(WORD x, WORD y) {
	if (g_points.size() < 4) {
		POINT pt = {x,y};
		g_points.push_back(pt );
	}
}

LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	PAINTSTRUCT ps;
	HDC hdc;

	switch (message) {

		case WM_CREATE:
			CreateWindow(_T("button"), _T("Reset"), WS_CHILD | WS_VISIBLE, 20,30,80,35, hWnd, 
						reinterpret_cast<HMENU>(RESET_BUTTON_ID), hInst, NULL);
			break;

		case WM_COMMAND:
				if (LOWORD(wParam) == RESET_BUTTON_ID) {
					g_points.clear();
					InvalidateRect(hWnd, NULL, TRUE);
				}
			break;

		case WM_LBUTTONDOWN:
			HandleClick( GET_X_LPARAM(lParam), GET_Y_LPARAM(lParam) );
			InvalidateRect(hWnd, NULL, FALSE);
			break;

		case WM_PAINT:
			hdc = BeginPaint(hWnd, &ps);
			DrawBezierCurve(hdc);
			EndPaint(hWnd, &ps);
			break;

		case WM_DESTROY:
			PostQuitMessage(0);
			break;

		default:
			return DefWindowProc(hWnd, message, wParam, lParam);
	}
	return 0;
}

